/*
   VEKTORKOLLEKTOR Main- / Joystick-controller
   Runs on an Arduino Nano
   Programmed with Arduino 1.8.13
   
   Hardware connected to this controller is:
   - Arduino Nano with plotter control software
   - Joystick
   - Alarm beacon
   - TFT display with SD card slot
   - Start Stop button
   
   ----------------------------------------------------------------------------
   "THE BEER-WARE LICENSE" (Revision 42):
   Niklas Roy wrote this file. As long as you retain this notice you
   can do whatever you want with this stuff. If we meet some day, and you think
   this stuff is worth it, you can buy me a beer in return :) Niklas
   ----------------------------------------------------------------------------
*/

#include <Wire.h>
#include <SPI.h>
#include <SD.h>
#include <TFT.h>
#include <EEPROM.h>

#define SD_CS 4
#define TFT_CS 10
#define DC 9
#define RST 8

#define START_STOP 2 /*Start-Stop button*/

#define J_U A3/*up*/
#define J_D A1/*down*/
#define J_L A4/*left*/
#define J_R A2/*right*/
#define J_A A0/*action*/

#define LIGHT_BEACON 5 /*sound connections going to MP3 Arduino UNO*/


TFT tft = TFT(TFT_CS, DC, RST);


int startStopCount = 0;

int xPos = 2;
int yPos = 2;
int PxPos = xPos;
int PyPos = yPos;

boolean sdFail = false;
unsigned int fileNumber = 0;
char fileName[13];

char tftText[20];

boolean startStop = true; /*Start-Stop button*/
boolean PstartStop = true;
boolean startDrawing = false;
boolean action = false;
boolean Paction = false;
int     lineCount = 0;


//=================================================================================== SETUP
void setup() {
  // ----------------------------------------------------- read eeprom
  int addr = 0;
  EEPROM.get(addr, fileNumber);
  //fileNumber=24;               // last file on SD card
  //writeEeprom();               // save file number in EEPROM

  // ----------------------------------------------------- serial
  Serial.begin(9600);

  // ----------------------------------------------------- tft
  tft.initR(INITR_BLACKTAB);
  tft.setRotation(1);
  tftClear();

  // ----------------------------------------------------- SD
  tft.setTextSize(1);
  tft.setTextColor(rgb(1, 1, 1));
  tft.setCursor(10, 10);
  if (init_SD()) {
    tft.print("SD OK!");
  } else {
    tft.print("SD FAIL!");
  }

  // ----------------------------------------------------- joystick / interface
  pinMode(J_U, INPUT_PULLUP);
  pinMode(J_D, INPUT_PULLUP);
  pinMode(J_L, INPUT_PULLUP);
  pinMode(J_R, INPUT_PULLUP);
  pinMode(J_A, INPUT_PULLUP);
  pinMode(START_STOP, INPUT_PULLUP);

  // ----------------------------------------------------- sound connection to Arduino UNO

  pinMode(LIGHT_BEACON, OUTPUT);

  digitalWrite(LIGHT_BEACON, HIGH);
  delay(2000);
  digitalWrite(LIGHT_BEACON, LOW);
  delay(500);

  startStop = !digitalRead(START_STOP);
  PstartStop = startStop;
}

//=================================================================================== LOOP
void loop() {

  PstartStop = startStop;
  PxPos = xPos;
  PyPos = yPos;
  Paction = action;

  if (!digitalRead(START_STOP)) { // start stop has been pushed
    startStopCount++;
  } else {
    startStopCount = 0;
  }

  startStop = startStopCount >= 5;


  if (startStop && !PstartStop) { /*start/stop has been pushed*/
    if (startDrawing == false) {
      startDrawing = true;         // start new drawing

      digitalWrite(LIGHT_BEACON, HIGH);
      delay(50);
      xPos = 2;
      yPos = 2;
      PxPos = 2;
      PyPos = 2;

      fileNumber++;                // increment file number
      writeEeprom();               // save file number in EEPROM
      lineCount = 0;

      tftClear();
      tft.setTextColor(rgb(1, 1, 1));
      tft.setCursor(10, 10);
      sprintf(tftText,  "START FILE: %04d", fileNumber);
      tft.print(tftText);

      SDwrite(3, 1, 1, 1);         // write "calibrate command"

      sendMessage(3, 1, 1, 1);     // calibrate
      sendMessage(1, 1, 50, 1);
      sendMessage(3, 1, 1, 1);
      sendMessage(1, 1, 1, 1);     // draw rectangular frame on paper
      sendMessage(1, 220, 1, 255);
      sendMessage(1, 220, 200, 255);
      sendMessage(1, 1, 200, 255);
      sendMessage(1, 1, 1, 255);
      sendMessage(1, 1, 1, 1);
      drawLayout();                // draw rectangle on screen
      tft.setTextColor(rgb(1, 1, 1));
      tft.setCursor(125, 10);
      sprintf(tftText,  "%03d", fileNumber);
      tft.println(tftText);

      digitalWrite(LIGHT_BEACON, LOW);
    } else {





      if (lineCount < 10) {
        fileNumber--;                // decrement file number
        writeEeprom();               // save file number in EEPROM

        digitalWrite(LIGHT_BEACON, HIGH);
        startDrawing = false;        // finish drawing
        tftClear();
        tft.setTextColor(rgb(1, 1, 1));
        tft.setCursor(10, 10);
        sprintf(tftText,  "NO FILE SAVED");
        tft.println(tftText);
        tft.setCursor(10, 30);
        sprintf(tftText,  "TOO FEW VECTORS!");
        tft.println(tftText);

      } else {
        digitalWrite(LIGHT_BEACON, HIGH);
        startDrawing = false;        // finish drawing
        tftClear();
        tft.setTextColor(rgb(1, 1, 1));
        tft.setCursor(10, 10);
        sprintf(tftText,  "FINISHED FILE: %04d", fileNumber);
        tft.println(tftText);
        tft.setCursor(10, 30);
        sprintf(tftText,  "VECTORS:       %04d", lineCount);
        tft.println(tftText);

      }

      SDwrite(1, 1, 200, 1);
      sendMessage(1, 1, 220, 1);   // move paper out
      sendMessage(3, 1, 1, 1);     // calibrate
      sendMessage(1, 1, 220, 1);   // move paper out

      xPos = 2;
      yPos = 2;
      PxPos = 2;
      PyPos = 2;
      digitalWrite(LIGHT_BEACON, LOW);
    }
  }

  if (startDrawing) { // only if in drawing mode
    if (!digitalRead(J_U))yPos -= 2; // check joystick directions -> calculate new pen position
    if (!digitalRead(J_D))yPos += 2;
    if (!digitalRead(J_L))xPos -= 2;
    if (!digitalRead(J_R))xPos += 2;

    xPos = constrain(xPos, 2, 218);
    yPos = constrain(yPos, 2, 198);

    byte penByte = 1;

    if (!digitalRead(J_A)) { // check ACTION button
      action = true;
      penByte = 255;
      tft.line(xPos / 2 + 10, yPos / 2 + 10, PxPos / 2 + 10, PyPos / 2 + 10);
    }
    sendMessage(1, xPos, yPos, penByte);

    if (PxPos != xPos || PyPos != yPos || action != Paction) {
      SDwrite(1, xPos, yPos, penByte);
      lineCount++;
    }
  }
}
